function [predicate_index,predicates_com_count]=AnalyzePredicates(predicates,predicate,bShow)
% [predicate_index,predicates_com_count]=AnalyzePredicates(predicates,predicate,bShow)
% Display a number of predicates and some statistics.
% The first input argument is a cell array containing a number of
% predicates found with the function FindCandidatePredicates.
% The second optional input argument is the predicate used for 
% authentication, i.e., the one to be broken.
% The output arguments include the index of the to-be-broken predicate and
% an array of statistics.
%
% Developer: Shujun Li @ www.hooklee.com, 2009.

% Index of the to-be-broken predicate.
predicate_index=0;
% Statistics of different components of the to-be-broken predicate.
predicates_com_count=zeros(4,2);

if nargin<3
    bShow=1;
end
if nargin<1
    disp('DisplayPredicates: at least one input parameter is needed!');
    return;
end

s=numel(predicates);

for i=1:s
    if bShow
        fprintf('Predicate %d: (%d,%d)%c; (%d,%d)%c\n',i,...
            predicates{i}(1,1:2),predicates{i}(1,3)+'A'-1,...
            predicates{i}(2,1:2),predicates{i}(2,3)+'A'-1);
    end
    if nargin>=2
        if all(all(predicates{i}==predicate))
            predicate_index=i;
        end
        mn1=(predicates{i}(1,1)==predicate(1,1) && predicates{i}(1,2)==predicate(1,2));
        H1=(predicates{i}(1,3)==predicate(1,3));
        mn2=(predicates{i}(2,1)==predicate(2,1) && predicates{i}(2,2)==predicate(2,2));
        H2=(predicates{i}(2,3)==predicate(2,3));
        if mn1
            predicates_com_count(1,1)=predicates_com_count(1,1)+1;
        end
        if H1
            predicates_com_count(1,2)=predicates_com_count(1,2)+1;
        end
        if mn2
            predicates_com_count(2,1)=predicates_com_count(2,1)+1;
        end
        if H2
            predicates_com_count(2,2)=predicates_com_count(2,2)+1;
        end
        if (mn1 && mn2)
            predicates_com_count(3,1)=predicates_com_count(3,1)+1;
        end
        if (H1 && H2)
            predicates_com_count(3,2)=predicates_com_count(3,2)+1;
        end
        if (mn1 && H1)
            predicates_com_count(4,1)=predicates_com_count(4,1)+1;
        end
        if (mn2 && H2)
            predicates_com_count(4,2)=predicates_com_count(4,2)+1;
        end
    end
end
if nargin>=2
    if predicate_index==0
        fprintf('The to-be-broken predicate is not in the set of valid predicates!');
    else
        fprintf('Predicate %d: (%d,%d)%c; (%d,%d)%c *****\n',predicate_index,...
          predicates{predicate_index}(1,1:2),predicates{predicate_index}(1,3)+'A'-1,...
            predicates{predicate_index}(2,1:2),predicates{predicate_index}(2,3)+'A'-1);
    end
    fprintf('Statistics: (%d,%d)(%d,%d) = %d; %c%c = %d\n',...
        predicate(1,1:2),predicate(2,1:2),predicates_com_count(3,1),...
        predicate(1,3)+'A'-1,predicate(2,3)+'A'-1,predicates_com_count(3,2));
    fprintf('Statistics: (%d,%d)%c = %d; (%d,%d)%c = %d\n',...
        predicate(1,1:2),predicate(1,3)+'A'-1,predicates_com_count(4,1),...
        predicate(2,1:2),predicate(1,3)+'A'-1,predicates_com_count(4,2));
    fprintf('Statistics: (%d,%d) = %d, %c = %d; (%d,%d) = %d, %c = %d\n',...
        predicate(1,1:2),predicates_com_count(1,1),predicate(1,3)+'A'-1,predicates_com_count(1,2),...
        predicate(2,1:2),predicates_com_count(2,1),predicate(2,3)+'A'-1,predicates_com_count(2,2));
end
