function predicates_stat=AnalyzePredicatesSets(varargin)
% predicates_stat=AnalyzePredicatesSets(predicates_sets,ln,m,n,H)
% predicates_stat=AnalyzePredicatesSets(predicates_sets,password,ln,m,n,H)
% The second input argument is the number of statistics that will be
% displayed (default value: 0)
% The output is a 1X2 cell array which includes the ranking statistics 
% of the cell index pairs and also on the secret character pairs appearing 
% in all input sets of predicates.
%
% Developer: Shujun Li @ www.hooklee.com, 2009.

predicates_stat=[];

if nargin<1
    disp('AnalyzePredicatesSets: at least one input argument is needed!');
    return;
end

predicates_sets=varargin{1};
password=[];
index=[2 3 4 5];
if nargin>=2
    if numel(varargin{2})>1 % password
        password=varargin{2};
        index=[3 4 5 6];
    end
end
if nargin<index(1)
    ln=0;
else
    ln=varargin{index(1)};
end
if nargin<index(2)
    m=5;
else
    m=varargin{index(2)};
end
if nargin<index(3)
    n=5;
else
    n=varargin{index(3)};
end
if nargin<index(4)
    H=26;
else
    H=varargin{index(4)};
end

% Use t (which <=na) to denote the number of observed sessions.
t=numel(predicates_sets);

ci_max_plus_1=m*n*m*n;
ci_max=ci_max_plus_1-1;
sc_max_plus_1=H*H;
sc_max=sc_max_plus_1-1;
predicates_stat=cell(1,2);
predicates_stat{1}=zeros(ci_max_plus_1,2);
predicates_stat{1}(:,1)=0:ci_max;
predicates_stat{2}=zeros(sc_max_plus_1,2);
predicates_stat{2}(:,1)=0:sc_max;

% Transform predicates to 0-based cell indices and secret characters, 
% and make the statistics.
for i=1:t
    predicates_set=predicates_sets{i};
    s=numel(predicates_set);
    for j=1:s
        predicate=predicates_set{j};
        cell_index_plus_1=Predicate2CellIndex(predicate,m,n)+1;
        predicates_stat{1}(cell_index_plus_1,2)=predicates_stat{1}(cell_index_plus_1,2)+1;
        secret_character_plus_1=Predicate2SecretCharacter(predicate,H)+1;
        predicates_stat{2}(secret_character_plus_1,2)=predicates_stat{2}(secret_character_plus_1,2)+1;
    end
end
% Sort the statistical results according to the occurrence numbers of cell
% indices and secret characters.
for i=1:2
    predicates_stat{i}(logical(predicates_stat{i}(:,2)==0),:)=[];
    [temp,index]=sort(predicates_stat{i}(:,2),'descend');
    predicates_stat{i}=predicates_stat{i}(index,:);
end
ci_number=size(predicates_stat{1},1);
sc_number=size(predicates_stat{2},1);

if ln>0
    ln_ci=min(ln,ci_number);
    ln_sc=min(ln,sc_number);
    fprintf('The first %d statistics of cell index pairs are as follows:\n',ln_ci);
    for i=1:ln_ci
        fprintf('No. %d: Cell Index Pair = %s; Count = %d',i,...
            Int2CellIndicesStr(predicates_stat{1}(i,1),m,n),predicates_stat{1}(i,2));
        if (~isempty(password) && predicates_stat{1}(i,1)==Predicate2CellIndex(password(1:2,1:2)))
            fprintf(' *****');
        end
        fprintf('\n');
    end
    fprintf('The first %d statistics of secret character pairs are as follows:\n',ln_sc);
    for i=1:ln_sc
        fprintf('No. %d: Secret Character Pair = %s; Count = %d',i,...
            Int2SecretCharacters(predicates_stat{2}(i,1),H),predicates_stat{2}(i,2));
        if ~isempty(password)
            temp=isInPassword(predicates_stat{2}(i,1),password,H);
            if temp>0 
                fprintf([' ' char('*'*ones(1,temp))]);
            end
        end
        fprintf('\n');
    end
end

function s=Int2CellIndicesStr(x,m,n)
% Transform an integer x representing a cell index pair to a string.
n2=rem(x,n)+1;
x=floor(x/n);
m2=rem(x,m)+1;
x=floor(x/m);
n1=rem(x,n)+1;
m1=floor(x/n)+1;
s=sprintf('(%d,%d)(%d,%d)',m1,n1,m2,n2);

function s=Int2SecretCharacters(x,H)
% Transform an integer x representing a secret character pair to a string.
s=sprintf('%c%c',floor(x/H)+'A',rem(x,H)+'A');

function y=isInPassword(x,password,H)
% Check is an integer x=H*H1+H2-1 belongs to the password.
y=0;
len_plus_2=size(password,2);
H1=floor(x/H)+1;
H2=rem(x,H)+1;
for i=3:len_plus_2
    if (H1==password(1,i) && H2==password(2,i))
        y=i-2;
        return;
    end
end

