function passwords=BreakPassword(predicates_sets,I,len,m,n)
% passwords=BreakPassword(predicates_sets,I,len,m,n)
% Try to (in most cases partially) break the password, based on the
% information observed in a number of authentication sessions.
% The output is a cell vector, whose element denotes a partially broken
% password, which is represented as a 1X(len+1) cell vector.
%
% Developer: Shujun Li @ www.hooklee.com, 2009.

passwords=[];

if nargin<2
    disp('BreakPassword: at least two input arguments are needed!');
    return;
end

if nargin<5
    n=5;
    if nargin<4
        m=5;
        if nargin<3
            len=10;
        end
    end
end

len_plus_1=len+1;
I_plus_1=I+1; % Adjust to the column index in the cell array "passwords".

na=numel(predicates_sets);

predicates_sets=RefinePredicatesSets(predicates_sets,I);

s=zeros(1,na);
for i=1:na
    s(i)=numel(predicates_sets{i});
end
index=find(s==1);
if ~isempty(index)
    % When the cell index pair can be uniquely determined by
    % RefinePredicatesSets(), set cell_indices_no1 to be a 1-D cell array.
    cell_indices_no1_number=1;
    cell_indices_no1(1)=Predicate2CellIndex(predicates_sets{index(1)}{1});
else
    % When the cell index pair can not be uniquely determined by
    % RefinePredicatesSets(), use AnalyzePredicatesSets() to get a ranking
    % and choose the cell index pair(s) ranked at the first position.
    predicates_stat=AnalyzePredicatesSets(predicates_sets);
    predicates_stat_size1=size(predicates_stat{1},1);
    counter=predicates_stat{1}(1,2);
    cell_indices_no1_number=1;
    for i=2:predicates_stat_size1
        if predicates_stat{1}(i,2)<counter
            cell_indices_no1_number=i-1;
            break;
        end
    end
    cell_indices_no1=zeros(1,cell_indices_no1_number);
    for i=1:cell_indices_no1_number
        cell_indices_no1(i)=predicates_stat{1}(i,1);
    end
end

cell_indices=cell(1,na);
% Transform each 2x2 cell index array into an integer for comparison.
for i=1:na
    cell_indices{i}=PredicatesSet2CellIndices(predicates_sets{i},m,n);
end

passwords=cell(1,cell_indices_no1_number);
for i=1:cell_indices_no1_number
    predicates_sets_refined=cell(1,na);
    for j=1:na
        predicates_sets_refined{j}=predicates_sets{j}(...
            logical(ismember(cell_indices{j},cell_indices_no1(i))));
    end
    % Further refine the predicate sets using information in I.
    predicates_sets_refined=RefinePredicatesSetsI(predicates_sets_refined,I);
    % Set the secret character pairs in the password.
    passwords{i}=cell(1,len_plus_1);
    passwords{i}{1}=Int2CellIndices(cell_indices_no1(i),m,n);
    for j=2:len_plus_1
        index=find(I_plus_1(1:na)==j);
        if isempty(index)
            continue;
        end
        predicates=predicates_sets_refined{index(1)};
        s=numel(predicates);
        passwords{i}{j}=zeros(2,s);
        for k=1:s
            passwords{i}{j}(:,k)=predicates{k}(:,3);
        end
    end
end

DisplayBrokenPasswords(passwords);

function ci=Int2CellIndices(x,m,n)
% Transform an integer representing a cell index pair to a 2X2 array
% representation of the cell index pair.
n2=rem(x,n)+1;
x=floor(x/n);
m2=rem(x,m)+1;
x=floor(x/m);
n1=rem(x,n)+1;
m1=floor(x/n)+1;
ci=[m1 n1;m2 n2];
