function rtable2=DisplayResponseTable(rtable)
% rtable2=DisplayResponseTable(rtable)
% Display a given response table.
% The output (optional) is a cell matrix in which each element is the
% two-character word instead of a number between 0 and 26^2.
%
% Developer: Shujun Li @ www.hooklee.com, 2009.

if nargin<1
    rtable=GenerateResponseTable;
end

rtable_size=size(rtable);

l2=rtable_size(1);
l=round(log2(l2));

if (numel(rtable_size)~=2 || rtable_size(1)~=rtable_size(2) || l2~=2^l)
    disp('DisplayResponseTable: invalid response table!');
    return;
end

yesno={'NO' 'YES'};
index=cell(1,l2);
for i=1:l2
    index{i}={yesno{bitget(i-1,l:-1:1)+1}};
end

fprintf([repmat(' ',1,l*3+2) '| ']);
for i=1:l2
    for j=1:l
        fprintf('%3s',index{i}{j});
        if j<l
            fprintf(' ');
        end
    end
    if i<l2
        fprintf(' | ');
    else
        fprintf('\n');
    end
end
fprintf([repmat('-',1,(l2+1)*l*3+4*l2+2) '\n']);
for i1=1:l2
    for j=1:l
        fprintf('%3s',index{i1}{j});
        if j<l
            fprintf(' ');
        else
            fprintf(' | ');
        end
    end
    for i2=1:l2
        fprintf('  %c %c  ',rtable{i1,i2});
        if i2<l2
            fprintf(' | ');
        else
            fprintf('\n');
        end
    end
end
