function rtable=GenerateResponseTable(l,p)
% rtable=GenerateResponseTable(l,p)
% Generate a response table with the parameters l and p.
% Currently, only p=2 is supported.
% The output is a (2^l X 2^l) cell matrix whose element is a two-character word.
%
% Developer: Shujun Li @ www.hooklee.com, 2009.

if nargin<2
    p=2;
    if nargin<1
        l=2;
    end
end

rtable=[];
if p~=2
    disp('GenerateResponseTable: p~=2 is not supported in this edition.');
    return;
end
if (l<1 || l>16)
    disp('GenerateResponseTable: l must be between 2 and 16!');
    return;
end

l2=2^l;

% generate a latin square
index_table=GenerateLatinSquare(l);

responses=randperm(26^2); % l2 different responses, each representing a two-character word
rtable_n=responses(index_table);
rtable=cell(l2);
for i1=1:l2
    for i2=1:l2
        rtable{i1,i2}=char([floor(rtable_n(i1,i2)/26)+'A' rem(rtable_n(i1,i2),26)+'A']);
    end
end
