function predicates_sets_refined=RefinePredicatesSets(varargin)
% predicates_sets_refined=RefinePredicatesSets(predicates_sets,I,bShow,m,n)
% predicates_sets_refined=RefinePredicatesSets(predicates_sets,I,predicates,bShow,m,n)
% Refine a number of sets of valid predicates, by comparing their cell indices.
% The input argument "predicates" is the real predicates that are to be broken.
% The input argument "bShow" controls if the results will be shown.
%
% Developer: Shujun Li @ www.hooklee.com, 2009.

predicates_sets_refined=[];

if nargin<2
    disp('RefinePredicatesSets: at least two input arguments are needed!');
    return;
end

predicates_sets=varargin{1};
I=varargin{2};
predicates=[];
index=[3 4 5];
if nargin>=3
    if iscell(varargin{3}) % predicates
        predicates=varargin{3};
        index=[4 5 6];
    end
end
if nargin<index(1)
    bShow=0;
else
    bShow=varargin{index(1)};
end
if nargin<index(2)
    m=5;
else
    m=varargin{index(2)};
end
if nargin<index(3)
    n=5;
else
    n=varargin{index(3)};
end

na=numel(predicates_sets);
if na==1
    predicates_sets_refined=predicates_sets;
    return;
end

cell_indices=cell(1,na);

% Transform each 2x2 cell index array into an integer.
for i=1:na
    cell_indices{i}=PredicatesSet2CellIndices(predicates_sets{i},m,n);
end

% Find indices of the intersection of all the predicate sets.
cell_indices_intersect=cell_indices{1};
for i=2:na
    cell_indices_intersect=intersect(cell_indices_intersect,cell_indices{i});
end

% Refine all the predicate sets by using the above intersection.
predicates_sets_refined=cell(1,na);
for i=1:na
    predicates_sets_refined{i}=predicates_sets{i}(...
        logical(ismember(cell_indices{i},cell_indices_intersect)));
end

% Further refine predicates_sets_refined by exploiting the information
% contained in I.
predicates_sets_refined=RefinePredicatesSetsI(predicates_sets_refined,I);

if bShow
    DisplayPredicatesSetAttack(predicates_sets_refined,predicates);
end
